@@ -17,11 +17,13 @@
 package Mail::SpamAssassin::DBBasedAddrList;
 
 use strict;
+use warnings;
 use bytes;
 use Fcntl;
 
 use Mail::SpamAssassin::PersistentAddrList;
 use Mail::SpamAssassin::Util;
+use Mail::SpamAssassin::Logger;
 
 use vars qw{
   @ISA
@@ -53,62 +55,55 @@ sub new_checker {
     'locked_file'	=> ''
   };
 
-  my $path;
-
   my @order = split (' ', $main->{conf}->{auto_whitelist_db_modules});
   my $dbm_module = Mail::SpamAssassin::Util::first_available_module (@order);
   if (!$dbm_module) {
-    die "Cannot find a usable DB package from auto_whitelist_db_modules: ".
+    die "auto-whitelist: cannot find a usable DB package from auto_whitelist_db_modules: " .
 	$main->{conf}->{auto_whitelist_db_modules}."\n";
   }
 
-  my $umask = umask 0;
-  if(defined($main->{conf}->{auto_whitelist_path})) # if undef then don't worry -- empty hash!
-  {
-    $path = $main->sed_path ($main->{conf}->{auto_whitelist_path});
+  # if undef then don't worry -- empty hash!
+  if (defined($main->{conf}->{auto_whitelist_path})) {
+    my $path = $main->sed_path($main->{conf}->{auto_whitelist_path});
+    my ($mod1, $mod2);
 
     if ($main->{locker}->safe_lock
-			($path, 30))
+                       ($path, 30, $main->{conf}->{auto_whitelist_file_mode}))
     {
       $self->{locked_file} = $path;
-      $self->{is_locked} = 1;
-      dbg("Tie-ing to DB file R/W in $path");
-      tie %{$self->{accum}},$dbm_module,$path,
-		  O_RDWR|O_CREAT,   #open rw w/lock
-		  (oct ($main->{conf}->{auto_whitelist_file_mode}) & 0666)
-	 or goto failed_to_tie;
-
-    } else {
-      $self->{is_locked} = 0;
-      dbg("Tie-ing to DB file R/O in $path");
-      tie %{$self->{accum}},$dbm_module,$path,
-		  O_RDONLY,         #open ro w/o lock
-		  (oct ($main->{conf}->{auto_whitelist_file_mode}) & 0666)
-	 or goto failed_to_tie;
+      $self->{is_locked}   = 1;
+      ($mod1, $mod2) = ('R/W', O_RDWR | O_CREAT);
     }
+    else {
+      $self->{is_locked} = 0;
+      ($mod1, $mod2) = ('R/O', O_RDONLY);
+    }
+
+    dbg("auto-whitelist: tie-ing to DB file of type $dbm_module $mod1 in $path");
+
+    if (! tie %{ $self->{accum} }, $dbm_module, $path, $mod2,
+      oct($main->{conf}->{auto_whitelist_file_mode}) ) {
+        my $err = $!;   # might get overwritten later
+        if ($self->{is_locked}) {
+          $self->{main}->{locker}->safe_unlock($self->{locked_file});
+          $self->{is_locked} = 0;
+        }
+        die "auto-whitelist: cannot open auto_whitelist_path $path: $err\n";
+      }
   }
-  umask $umask;
 
   bless ($self, $class);
   return $self;
-
-failed_to_tie:
-  umask $umask;
-  if ($self->{is_locked}) {
-    $self->{main}->{locker}->safe_unlock ($self->{locked_file});
-    $self->{is_locked} = 0;
-  }
-  die "Cannot open auto_whitelist_path $path: $!\n";
 }
 
 ###########################################################################
 
 sub finish {
   my $self = shift;
-  dbg("DB addr list: untie-ing and unlocking.");
+  dbg("auto-whitelist: DB addr list: untie-ing and unlocking");
   untie %{$self->{accum}};
   if ($self->{is_locked}) {
-    dbg ("DB addr list: file locked, breaking lock.");
+    dbg("auto-whitelist: DB addr list: file locked, breaking lock");
     $self->{main}->{locker}->safe_unlock ($self->{locked_file});
     $self->{is_locked} = 0;
   }
@@ -127,7 +122,7 @@ sub get_addr_entry {
   $entry->{count} = $self->{accum}->{$addr} || 0;
   $entry->{totscore} = $self->{accum}->{$addr.'|totscore'} || 0;
 
-  dbg ("auto-whitelist (db-based): $addr scores ".$entry->{count}.'/'.$entry->{totscore});
+  dbg("auto-whitelist: db-based $addr scores ".$entry->{count}.'/'.$entry->{totscore});
   return $entry;
 }
 
@@ -142,7 +137,7 @@ sub add_score {
     $entry->{count}++;
     $entry->{totscore} += $score;
 
-    dbg("add_score: New count: ".$entry->{count}.", new totscore: ".$entry->{totscore});
+    dbg("auto-whitelist: add_score: new count: ".$entry->{count}.", new totscore: ".$entry->{totscore});
 
     $self->{accum}->{$entry->{addr}} = $entry->{count};
     $self->{accum}->{$entry->{addr}.'|totscore'} = $entry->{totscore};
@@ -163,17 +158,16 @@ sub remove_entry {
     # try to delete any per-IP entries for this addr as well.
     # could be slow...
     my $mailaddr = $1;
-    my @keys = grep { /^\Q${mailaddr}\E\|ip=\d+\.\d+$/ }
-					keys %{$self->{accum}};
-    foreach my $key (@keys) {
-      delete $self->{accum}->{$key};
-      delete $self->{accum}->{$key.'|totscore'};
+
+    while (my ($key, $value) = each %{$self->{accum}}) {
+      # regex will catch both key and key|totscore entries and delete them
+      if ($key =~ /^\Q${mailaddr}\E\|/) {
+        delete $self->{accum}->{$key};
+      }
     }
   }
 }
 
 ###########################################################################
 
-sub dbg { Mail::SpamAssassin::dbg (@_); }
-
 1;
