@@ -17,11 +17,13 @@
 package Mail::SpamAssassin::Locker::Flock;
 
 use strict;
+use warnings;
 use bytes;
 
 use Mail::SpamAssassin;
 use Mail::SpamAssassin::Locker;
 use Mail::SpamAssassin::Util;
+use Mail::SpamAssassin::Logger;
 use File::Spec;
 use IO::File;
 use Fcntl qw(:DEFAULT :flock);
@@ -44,39 +46,44 @@ sub new {
 # Attempt to create a file lock, using NFS-UNsafe locking techniques.
 
 sub safe_lock {
-  my ($self, $path, $max_retries) = @_;
+  my ($self, $path, $max_retries, $mode) = @_;
   my $is_locked = 0;
   my @stat;
 
   $max_retries ||= 30;
+  $mode ||= 0600;
+  $mode = oct $mode if $mode =~ /^0/;   # accept number or string
 
   my $lock_file = "$path.mutex";
-  my $umask = umask 077;
+  my $umask = umask(~$mode);
   my $fh = new IO::File();
 
-  if (!$fh->open ("$lock_file", O_RDWR|O_CREAT)) {
+  if (!$fh->open ($lock_file, O_RDWR|O_CREAT)) {
       umask $umask; # just in case
-      die "lock: $$ cannot create lockfile $lock_file: $!\n";
+      die "locker: safe_lock: cannot create lockfile $lock_file: $!\n";
   }
   umask $umask; # we've created the file, so reset umask
 
-  dbg("lock: $$ created $lock_file");
+  dbg("locker: safe_lock: created $lock_file");
 
   my $unalarmed = 0;
+  my $oldalarm = 0;
+
   # use a SIGALRM-based timer -- more efficient than second-by-second
   # sleeps
   eval {
     local $SIG{ALRM} = sub { die "alarm\n" };
-    dbg("lock: $$ trying to get lock on $path with $max_retries timeout");
+    dbg("locker: safe_lock: trying to get lock on $path with $max_retries timeout");
 
     # max_retries is basically seconds! so use it for the timeout
-    alarm($max_retries);
+    $oldalarm = alarm $max_retries;
 
     # HELLO!?! IO::File doesn't have a flock() method?!
     if (flock ($fh, LOCK_EX)) {
-      alarm(0) and $unalarmed = 1; # avoid calling alarm(0) twice
+      alarm $oldalarm;
+      $unalarmed = 1; # avoid calling alarm(0) twice
 
-      dbg("lock: $$ link to $lock_file: link ok");
+      dbg("locker: safe_lock: link to $lock_file: link ok");
       $is_locked = 1;
 
       # just to be nice: let people know when it was locked
@@ -90,12 +97,14 @@ sub safe_lock {
     }
   };
 
-  $unalarmed or alarm(0); # if we die'd above, need to reset here
-  if ($@) {
-    if ($@ =~ /alarm/) {
-      dbg ("lock: $$ timed out after $max_retries secs.");
+  my $err = $@;
+
+  $unalarmed or alarm $oldalarm; # if we die'd above, need to reset here
+  if ($err) {
+    if ($err =~ /alarm/) {
+      dbg("locker: safe_lock: timed out after $max_retries seconds");
     } else {
-      die $@;
+      die "locker: safe_lock: $err";
     }
   }
 
@@ -108,7 +117,7 @@ sub safe_unlock {
   my ($self, $path) = @_;
 
   if (!exists $self->{lock_fhs} || !defined $self->{lock_fhs}->{$path}) {
-    dbg ("unlock: $$ no lock handle for $path - already unlocked?");
+    dbg("locker: safe_unlock: no lock handle for $path - already unlocked?");
     return;
   }
 
@@ -118,7 +127,7 @@ sub safe_unlock {
   flock ($fh, LOCK_UN);
   $fh->close();
 
-  dbg("unlock: $$ unlocked $path.mutex");
+  dbg("locker: safe_unlock: unlocked $path.mutex");
 
   # do NOT unlink! this would open a race, whereby:
   #
@@ -144,7 +153,7 @@ sub refresh_lock {
   return unless $path;
 
   if (!exists $self->{lock_fhs} || !defined $self->{lock_fhs}->{$path}) {
-    warn "refresh_lock: $$ no lock handle for $path\n";
+    warn "locker: refresh_lock: no lock handle for $path\n";
     return;
   }
 
@@ -152,11 +161,9 @@ sub refresh_lock {
   $fh->print ("$$\n");
   $fh->flush ();
 
-  dbg("refresh: $$ refresh $path.mutex");
+  dbg("locker: refresh_lock: refresh $path.mutex");
 }
 
 ###########################################################################
 
-sub dbg { Mail::SpamAssassin::dbg (@_); }
-
 1;
